SystemOrganization addCategory: #'Multilingual-TextConversion'!

StandardFileStream subclass: #MultiByteFileStream
	instanceVariableNames: 'converter lineEndConvention wantsLineEndConversion'
	classVariableNames: 'Cr CrLf Lf LineEndDefault LineEndStrings LookAheadCount'
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!MultiByteFileStream commentStamp: '<historical>' prior: 0!
The central class to access the external file.  The interface of this object is similar to good old StandardFileStream, but internally it asks the converter, which is a sub-instance of TextConverter, and do the text conversion.

  It also combined the good old CrLfFileStream.  CrLfFileStream class>>new now returns an instance of MultiByteFileStream.

  There are several pitfalls:

  * You always have to be careful about the binary/text distinction.  In #text mode, it usually interpret the bytes.
  * A few file pointer operations treat the file as uninterpreted byte no matter what.  This means that if you use 'fileStream skip: -1', 'fileStream position: x', etc. in #text mode, the file position can be in the middle of multi byte character.  If you want to implement some function similar to #peek for example, call the saveStateOf: and restoreStateOf: methods to be able to get back to the original state.
  * #lineEndConvention: and #wantsLineEndConversion: (and #binary) can cause some puzzling situation because the inst var lineEndConvention and wantsLineEndConversion are mutated.  If you have any suggestions to clean up the protocol, please let me know.!

!MultiByteFileStream class methodsFor: 'class initialization' stamp: 'yo 2/21/2004 02:45'!
defaultToCR

	"MultiByteFileStream defaultToCR"
	LineEndDefault := #cr.
! !

!MultiByteFileStream class methodsFor: 'class initialization' stamp: 'yo 2/21/2004 02:45'!
defaultToCRLF

	"MultiByteFileStream defaultToCRLF"
	LineEndDefault := #crlf.! !

!MultiByteFileStream class methodsFor: 'class initialization' stamp: 'yo 2/21/2004 02:46'!
defaultToLF

	"MultiByteFileStream defaultToLF"
	LineEndDefault := #lf.
! !

!MultiByteFileStream class methodsFor: 'class initialization' stamp: 'JavierPimas 9/30/2011 16:49'!
guessDefaultLineEndConvention
	"Lets try to guess the line end convention from what we know about the
	path name delimiter from FileDirectory."
	FileDirectory pathNameDelimiter = $:
		ifTrue: [^ self defaultToCR].
	FileDirectory pathNameDelimiter = $/
		ifTrue: [^ (OSPlatform isMacOSX or: [OSPlatform isUnix])
				ifTrue: [ self defaultToLF]
				ifFalse: [ self defaultToCR]
		].
	FileDirectory pathNameDelimiter = $\
		ifTrue: [^ self defaultToCRLF].
	"in case we don't know"
	^ self defaultToCR! !

!MultiByteFileStream class methodsFor: 'class initialization' stamp: 'yo 2/21/2004 02:44'!
initialize

	"MultiByteFileStream initialize"
	Cr := Character cr.
	Lf := Character lf.
	CrLf := String with: Cr with: Lf.
	LineEndStrings := Dictionary new.
	LineEndStrings at: #cr put: (String with: Character cr).
	LineEndStrings at: #lf put: (String with: Character lf).
	LineEndStrings at: #crlf put: (String with: Character cr with: Character lf).
	LookAheadCount := 2048.
	Smalltalk addToStartUpList: self.
	self startUp.
! !

!MultiByteFileStream class methodsFor: 'accessing' stamp: 'StephaneDucasse 10/18/2010 14:05'!
lineEndDefault
	"Answer the default line-ending convention that will be used by default, which was determined at start-up by looking at platform attributes."
	^ LineEndDefault
! !

!MultiByteFileStream class methodsFor: 'instance creation' stamp: 'yo 8/28/2002 11:43'!
newFrom: aFileStream

	| rw n |
	n := aFileStream name.
	rw := aFileStream isReadOnly not.
	aFileStream close.
	^self new open: n forWrite: rw.
! !

!MultiByteFileStream class methodsFor: 'class initialization' stamp: 'yo 2/21/2004 02:44'!
startUp

	self guessDefaultLineEndConvention.
! !

!MultiByteFileStream methodsFor: 'remnant' stamp: 'yo 8/28/2002 11:06'!
accepts: aSymbol

 	^ converter accepts: aSymbol.
! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'yo 2/21/2004 02:57'!
ascii

	super ascii.
	self detectLineEndConvention.
! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'yo 2/24/2004 13:38'!
bareNext

	 ^ self converter nextFromStream: self.
! !

!MultiByteFileStream methodsFor: 'fileIn/Out' stamp: 'HenrikSperreJohansen 2/9/2010 11:23'!
basicChunk
	"If our buffer in collection contains an chunk with no embedded !!'s, nor
	any non-ascii characters, return that.
	This presumes the source code encoding is unambiguously ascii-compatible"
	| bufferIX goodString |
	"Not possible if read buffering disabled"
	collection ifNil: [^nil].
	^ ((bufferIX := (collection indexOf: $!! startingAt: position + 1) min: readLimit +1) > 0
			and: [bufferIX < collection size
					and: [(collection at: bufferIX + 1)
								~= $!!
							and: [goodString := collection copyFrom: position + 1 to: bufferIX - 1.
								goodString isAsciiString]]])
		ifTrue: [ position := bufferIX.
				goodString]! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'md 10/17/2004 16:09'!
basicNext: anInteger

	^ super next: anInteger.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:07'!
basicNext: n into: aString

	^ super next: n into: aString.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:07'!
basicNextInto: aString

	^ super nextInto: aString.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:07'!
basicNextPut: char

	^ super nextPut: char.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:07'!
basicNextPutAll: aString

	^ super nextPutAll: aString.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:07'!
basicPeek

	^ super peek
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:08'!
basicPosition

	^ super position.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:08'!
basicPosition: pos

	^ super position: pos.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:08'!
basicReadInto: byteArray startingAt: startIndex count: count

	^ super readInto: byteArray startingAt: startIndex count: count.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:08'!
basicSetToEnd

	^ super setToEnd.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:08'!
basicSkip: n

	^ super skip: n.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:08'!
basicUpTo: delim

	^ super upTo: delim.
! !

!MultiByteFileStream methodsFor: 'private basic' stamp: 'yo 8/28/2002 11:09'!
basicVerbatim: aString

	^ super verbatim: aString.
! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'nice 5/10/2009 00:14'!
binary

	super binary.
	self lineEndConvention: nil! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'yo 2/21/2004 02:56'!
convertStringFromCr: aString 
	| inStream outStream |
	lineEndConvention ifNil: [^ aString].
	lineEndConvention == #cr ifTrue: [^ aString].
	lineEndConvention == #lf ifTrue: [^ aString copy replaceAll: Cr with: Lf].
	"lineEndConvention == #crlf"
	inStream := aString readStream.
	outStream :=  (String new: aString size) writeStream.
	[inStream atEnd]
		whileFalse: 
			[outStream nextPutAll: (inStream upTo: Cr).
			(inStream atEnd not or: [aString last = Cr])
				ifTrue: [outStream nextPutAll: CrLf]].
	^ outStream contents! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'yo 2/21/2004 02:56'!
convertStringToCr: aString 
	| inStream outStream |
	lineEndConvention ifNil: [^ aString].
	lineEndConvention == #cr ifTrue: [^ aString].
	lineEndConvention == #lf ifTrue: [^ aString copy replaceAll: Lf with: Cr].
	"lineEndConvention == #crlf"
	inStream := aString readStream.
	outStream := (String new: aString size) writeStream.
	[inStream atEnd]
		whileFalse: 
			[outStream nextPutAll: (inStream upTo: Cr).
			(inStream atEnd not or: [aString last = Cr])
				ifTrue: 
					[outStream nextPut: Cr.
					inStream peek = Lf ifTrue: [inStream next]]].
	^ outStream contents! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'nice 5/10/2009 00:17'!
converter

	converter ifNil: [self converter: TextConverter defaultSystemConverter].
	^ converter
! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'nice 5/10/2009 00:18'!
converter: aConverter

	converter := aConverter.
	self installLineEndConventionInConverter
! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'tbn 5/10/2012 15:29'!
detectLineEndConvention
	"Detect the line end convention used in this stream. The result may be either #cr, #lf or #crlf."
	| char numRead state |
	self isBinary ifTrue: [^ self error: 'Line end conventions are not used on binary streams'].
	wantsLineEndConversion == true ifFalse: [self lineEndConvention: nil.
					^lineEndConvention].
	self closed ifTrue: [self lineEndConvention: LineEndDefault.
					^lineEndConvention].

	"Default if nothing else found"
	numRead := 0.
	state := self converter saveStateOf: self.
	lineEndConvention := nil.
	[super atEnd not and: [numRead < LookAheadCount]]
		whileTrue: 
			[char := self next.
			char = Lf
				ifTrue: 
					[converter restoreStateOf: self with: state.
					self lineEndConvention: #lf.
					^lineEndConvention].
			char = Cr
				ifTrue: 
					[self peek = Lf
						ifTrue: [self lineEndConvention: #crlf]
						ifFalse: [self lineEndConvention: #cr].
					converter restoreStateOf: self with: state.
					^ lineEndConvention].
			numRead := numRead + 1].
	converter restoreStateOf: self with: state.
	self lineEndConvention: LineEndDefault.
	^ lineEndConvention! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'ul 11/25/2009 01:22'!
doConversion

	^wantsLineEndConversion == true and: [ lineEndConvention notNil ]! !

!MultiByteFileStream methodsFor: 'fileIn/Out' stamp: 'yo 8/17/2004 10:03'!
fileIn

	self setConverterForCode.
	super fileIn.
! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'yo 8/6/2003 11:56'!
fileInEncodingName: aString

	self converter: (TextConverter newForEncoding: aString).
	super fileIn.
! !

!MultiByteFileStream methodsFor: 'remnant' stamp: 'yo 8/28/2002 11:09'!
filterFor: aFileStream

	| rw |
	name := aFileStream name.
	rw := aFileStream isReadOnly not.
	aFileStream close.
	self open: name forWrite: rw.
	^self.
! !

!MultiByteFileStream methodsFor: 'private' stamp: 'ul 11/25/2009 01:28'!
installLineEndConventionInConverter

	converter ifNotNil: [
		converter installLineEndConvention: (
			(wantsLineEndConversion == true and: [ lineEndConvention notNil ]) "#doConversion is inlined here"
				ifTrue: [ LineEndStrings at: lineEndConvention ]
				ifFalse: [ nil ]) ]! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'nk 9/5/2004 12:57'!
lineEndConvention

	^lineEndConvention! !

!MultiByteFileStream methodsFor: 'accessing' stamp: 'nice 11/28/2009 13:06'!
lineEndConvention: aSymbol

	(lineEndConvention := aSymbol) ifNotNil: [wantsLineEndConversion := true].
	self installLineEndConventionInConverter! !

!MultiByteFileStream methodsFor: 'public' stamp: 'ul 12/3/2009 01:33'!
next

	| char secondChar state |
	char := (converter ifNil: [ self converter ]) nextFromStream: self.
	(wantsLineEndConversion == true and: [ lineEndConvention notNil ]) "#doConversion is inlined here"
		 ifTrue: [
			char == Cr ifTrue: [
				state := converter saveStateOf: self.
				secondChar := self bareNext.
				secondChar ifNotNil: [
					secondChar == Lf ifFalse: [ converter restoreStateOf: self with: state ] ].
				^Cr ].
			char == Lf ifTrue: [
				^Cr ] ].
	^char.

! !

!MultiByteFileStream methodsFor: 'public' stamp: 'ul 11/25/2009 01:28'!
next: anInteger 

	| multiString |
	self isBinary ifTrue: [^ super next: anInteger].
	multiString := String new: anInteger.
	1 to: anInteger do: [:index |
		| character |
		(character := self next)
			ifNotNil: [ multiString at: index put: character ]
			ifNil: [
				multiString := multiString copyFrom: 1 to: index - 1.
				(wantsLineEndConversion == true and: [ lineEndConvention notNil ]) "#doConversion is inlined here"
					ifFalse: [ ^multiString ].
				^self next: anInteger innerFor: multiString ] ].
	(wantsLineEndConversion == true and: [ lineEndConvention notNil ]) "#doConversion is inlined here"
		 ifFalse: [ ^multiString ].

	multiString := self next: anInteger innerFor: multiString.
	(multiString size = anInteger or: [self atEnd]) ifTrue: [ ^ multiString].
	^ multiString, (self next: anInteger - multiString size).
! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'yo 2/24/2004 13:44'!
next: n innerFor: aString

	| peekChar state |
	"if we just read a CR, and the next character is an LF, then skip the LF"
	aString size = 0 ifTrue: [^ aString].
	(aString last = Character cr) ifTrue: [
		state := converter saveStateOf: self.
		peekChar := self bareNext.		"super peek doesn't work because it relies on #next"
		(peekChar notNil and: [peekChar ~= Character lf]) ifTrue: [
			converter restoreStateOf: self with: state.
		].
	].
 
	^ aString withSqueakLineEndings.
! !

!MultiByteFileStream methodsFor: 'fileIn/Out' stamp: 'JohanBrichau 6/11/2010 15:34'!
nextChunk
	"Answer the contents of the receiver, up to the next terminator
	character. Doubled terminators indicate an embedded terminator
	character."
	| bufferIX goodString |
	self skipSeparators.
	^ self
		parseLangTagFor: (self basicChunk
				ifNil: [String
						new: 1000
						streamContents: [:stream | 
							| character state |
							[(character := self next) == nil
								or: [character == $!!
										and: [state := converter saveStateOf: self..
											self next ~~ $!!]]]
								whileFalse: [stream nextPut: character].
							character
								ifNotNil: [converter restoreStateOf: self with: state.]]])! !

!MultiByteFileStream methodsFor: 'public' stamp: 'yo 2/21/2004 03:26'!
nextDelimited: terminator

	| out ch save |
	out := (String new: 1000) writeStream.
	self atEnd ifTrue: [^ ''].
	save := converter saveStateOf: self.

	self next = terminator ifFalse: [
		"absorb initial terminator"
		converter restoreStateOf: self with: save.
	].
	[(ch := self next) == nil] whileFalse: [
		(ch = terminator) ifTrue: [
			self peek = terminator ifTrue: [
				self next.  "skip doubled terminator"
			] ifFalse: [
				^ out contents  "terminator is not doubled; we're done!!"
			].
		].
		out nextPut: ch.
	].
	^ out contents.
! !

!MultiByteFileStream methodsFor: 'public' stamp: 'yo 8/28/2002 11:13'!
nextMatchAll: aColl

    | save |
    save := converter saveStateOf: self.
    aColl do: [:each |
       (self next) = each ifFalse: [
            converter restoreStateOf: self with: save.
            ^ false.
		].
	].
    ^ true.
! !

!MultiByteFileStream methodsFor: 'public' stamp: 'LukasRenggli 5/8/2010 18:41'!
nextPut: aCharacter
	aCharacter isInteger 
		ifTrue: [ ^ super nextPut: aCharacter ].
	(wantsLineEndConversion == true and: [ lineEndConvention notNil ]) "#doConversion is inlined here"
		 ifTrue: [
			aCharacter = Cr
				ifTrue: [ converter nextPutAll: (LineEndStrings at: lineEndConvention) toStream: self ]
				ifFalse: [ converter nextPut: aCharacter toStream: self ].
			^aCharacter ].
	^ self converter nextPut: aCharacter toStream: self
! !

!MultiByteFileStream methodsFor: 'public' stamp: 'LukasRenggli 5/8/2010 18:41'!
nextPutAll: aCollection
	^ (self isBinary or: [ aCollection class == ByteArray ])
		ifTrue: [ super nextPutAll: aCollection ]
		ifFalse: [ self converter nextPutAll: aCollection toStream: self ]! !

!MultiByteFileStream methodsFor: 'open/close' stamp: 'HenrikSperreJohansen 11/20/2009 15:13'!
open: fileName forWrite: writeMode 
	| result |
	result := super open: fileName forWrite: writeMode.
	result ifNotNil: [
			converter ifNil: [self converter: UTF8TextConverter new].
			lineEndConvention ifNil: [ self detectLineEndConvention ]
	].
	^result! !

!MultiByteFileStream methodsFor: 'public' stamp: 'yo 2/21/2004 04:00'!
peek
	"Answer what would be returned if the message next were sent to the receiver. If the receiver is at the end, answer nil.  "

	| next save |
	self atEnd ifTrue: [^ nil].
	save := converter saveStateOf: self.
	next := self next.
	converter restoreStateOf: self with: save.
	^ next.

! !

!MultiByteFileStream methodsFor: 'public' stamp: 'yo 8/28/2002 11:15'!
peekFor: item 

	| next state |
	"self atEnd ifTrue: [^ false]. -- SFStream will give nil"
	state := converter saveStateOf: self.
	(next := self next) == nil ifTrue: [^ false].
	item = next ifTrue: [^ true].
	converter restoreStateOf: self with: state.
	^ false.
! !

!MultiByteFileStream methodsFor: 'private' stamp: 'StephaneDucasse 5/3/2010 22:59'!
requestDropStream: dropIndex
	"Needs to install proper converter"
	
	| result |
	result := super requestDropStream: dropIndex.
	result ifNotNil: [
		converter ifNil: [self converter: UTF8TextConverter new].
		lineEndConvention ifNil: [ self detectLineEndConvention] ].
	^result! !

!MultiByteFileStream methodsFor: 'open/close' stamp: 'nice 5/10/2009 00:18'!
reset

	super reset.
	converter ifNil: [
		self converter: UTF8TextConverter new.
	].
! !

!MultiByteFileStream methodsFor: 'private' stamp: 'nice 1/18/2010 13:42'!
setConverterForCode

	| current |
	(SourceFiles at: 2)
		ifNotNil: [self fullName = (SourceFiles at: 2) fullName ifTrue: [^ self]].
	current := self converter saveStateOf: self.
	self position: 0.
	self binary.
	((self next: 3) = #[ 16rEF 16rBB 16rBF ]) ifTrue: [
		self converter: UTF8TextConverter new
	] ifFalse: [
		self converter: MacRomanTextConverter new.
	].
	converter restoreStateOf: self with: current.
	self text.
! !

!MultiByteFileStream methodsFor: 'public' stamp: 'ul 12/7/2009 23:35'!
skipSeparators

	| state character |
	[ 
		state := converter saveStateOf: self.
		(character := self next) 
			ifNil: [ false ]
			ifNotNil: [ character isSeparator ] ] whileTrue.
	character ifNotNil: [ 
		converter restoreStateOf: self with: state ]! !

!MultiByteFileStream methodsFor: 'public' stamp: 'ul 12/8/2009 00:05'!
skipSeparatorsAndPeekNext
	"Same as #skipSeparators, but returns the next character after the separators if such exists."

	| state character |
	[ 
		state := converter saveStateOf: self.
		(character := self next) 
			ifNil: [ false ]
			ifNotNil: [ character isSeparator ] ] whileTrue.
	character ifNotNil: [ 
		converter restoreStateOf: self with: state.
		^character ].
	! !

!MultiByteFileStream methodsFor: 'public' stamp: 'ul 12/4/2009 19:08'!
upTo: delimiter

	^self collectionSpecies new: 1000 streamContents: [ :stream |
		| ch |
		[ (ch := self next) == nil or: [ ch = delimiter ] ] 
			whileFalse: [ stream nextPut: ch ] ]
! !

!MultiByteFileStream methodsFor: 'public' stamp: 'nice 12/7/2009 08:26'!
upToAnyOf: delimiters do: aBlock

	^self collectionSpecies new: 1000 streamContents: [ :stream |
		| ch |
		[ (ch := self next) == nil or: [ (delimiters includes: ch) and: [aBlock value: ch. true] ] ] 
			whileFalse: [ stream nextPut: ch ] ]! !

!MultiByteFileStream methodsFor: 'public' stamp: 'ul 12/6/2009 04:18'!
upToEnd

	^self collectionSpecies
		new: self size - self position
		streamContents: [ :stream |
			| element |
			[ (element := self next) == nil ] whileFalse: [
				stream nextPut: element ] ]
! !

!MultiByteFileStream methodsFor: 'crlf private' stamp: 'ul 11/25/2009 01:22'!
wantsLineEndConversion

	^wantsLineEndConversion == true
! !

!MultiByteFileStream methodsFor: 'remnant' stamp: 'kph 3/1/2009 15:50'!
wantsLineEndConversion: aBoolean
	
	wantsLineEndConversion :=  aBoolean.

	lineEndConvention ifNil: [ self detectLineEndConvention ]. ! !

!Character methodsFor: '*Multilingual-TextConversion' stamp: 'michael.rueger 2/5/2009 17:02'!
macRomanToUnicode
	"Convert the receiver from MacRoman Unicode."

	^MacRomanTextConverter new unicodeToByte: self! !

!Character methodsFor: '*Multilingual-TextConversion' stamp: 'michael.rueger 2/5/2009 17:01'!
unicodeToMacRoman
	"Convert the receiver from Unicode to MacRoman encoding."

	^MacRomanTextConverter new byteToUnicode: self! !

ReadWriteStream subclass: #MultiByteBinaryOrTextStream
	instanceVariableNames: 'isBinary converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!MultiByteBinaryOrTextStream commentStamp: '<historical>' prior: 0!
It is similar to MultiByteFileStream, but works on in memory stream.!

!MultiByteBinaryOrTextStream class methodsFor: 'defaults' stamp: 'yo 2/25/2005 20:04'!
defaultConverter
	^ Latin1TextConverter new.
! !

!MultiByteBinaryOrTextStream class methodsFor: 'instance creation' stamp: 'ykoubo 9/28/2003 19:59'!
on: aCollection encoding: encodingName 
	| aTextConverter |
	encodingName isNil
		ifTrue: [aTextConverter := TextConverter default]
		ifFalse: [aTextConverter := TextConverter newForEncoding: encodingName].
	^ (self on: aCollection)
		converter: aTextConverter! !

!MultiByteBinaryOrTextStream class methodsFor: 'instance creation' stamp: 'yo 11/23/2003 20:32'!
with: aCollection encoding: encodingName 
	| aTextConverter |
	encodingName isNil
		ifTrue: [aTextConverter := TextConverter default]
		ifFalse: [aTextConverter := TextConverter newForEncoding: encodingName].
	^ (self with: aCollection)
		converter: aTextConverter! !

!MultiByteBinaryOrTextStream methodsFor: 'converting' stamp: 'yo 11/11/2002 13:16'!
asBinaryOrTextStream

	^ self
! !

!MultiByteBinaryOrTextStream methodsFor: 'accessing' stamp: 'yo 11/11/2002 13:16'!
ascii
	isBinary := false
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 16:01'!
basicNext

	^ super next 
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'md 10/20/2004 15:32'!
basicNext: anInteger

	^ super next: anInteger.
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicNext: n into: aString

	^ super next: n into: aString.
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicNextInto: aString

	^ super nextInto: aString.
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicNextPut: char

	^ super nextPut: char.
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicNextPutAll: aString

	^ super nextPutAll: aString.
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicPeek

	^ super peek
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicPosition

	^ super position.
! !

!MultiByteBinaryOrTextStream methodsFor: 'private basic' stamp: 'yo 11/11/2002 13:21'!
basicPosition: pos

	^ super position: pos.
! !

!MultiByteBinaryOrTextStream methodsFor: 'accessing' stamp: 'yo 11/11/2002 13:16'!
binary
	isBinary := true
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 7/30/2004 06:59'!
contents

	| ret state |
	state := converter saveStateOf: self.
	ret := self upToEnd.
	converter restoreStateOf: self with: state.
	^ ret.
! !

!MultiByteBinaryOrTextStream methodsFor: 'accessing' stamp: 'nk 8/2/2004 17:02'!
converter

	converter ifNil: [converter := self class defaultConverter].
	^ converter
! !

!MultiByteBinaryOrTextStream methodsFor: 'accessing' stamp: 'yo 8/7/2003 09:12'!
converter: aConverter

	converter := aConverter.
! !

!MultiByteBinaryOrTextStream methodsFor: 'filein/out' stamp: 'yo 8/17/2004 10:02'!
fileIn

	self setConverterForCode.
	super fileIn.
! !

!MultiByteBinaryOrTextStream methodsFor: 'filein/out' stamp: 'yo 11/11/2002 16:31'!
fileInObjectAndCode
	"This file may contain:
1) a fileIn of code  
2) just an object in SmartReferenceStream format 
3) both code and an object.
	File it in and return the object.  Note that self must be a FileStream or RWBinaryOrTextStream.  Maybe ReadWriteStream incorporate RWBinaryOrTextStream?"
	| refStream object |
	self text.
	self peek asciiValue = 4
		ifTrue: [  "pure object file"
			self binary.
			refStream := SmartRefStream on: self.
			object := refStream nextAndClose]
		ifFalse: [  "objects mixed with a fileIn"
			self fileIn.  "reads code and objects, then closes the file"
			self binary.
			object := SmartRefStream scannedObject].	"set by side effect of one of the chunks"
	SmartRefStream scannedObject: nil.  "clear scannedObject"
	^ object! !

!MultiByteBinaryOrTextStream methodsFor: 'private' stamp: 'StephaneDucasse 2/3/2010 22:18'!
guessConverter
	^ (self originalContents includesSubString: #[27 36] asString)
		ifTrue: [CompoundTextConverter new]
		ifFalse: [self class defaultConverter ]! !

!MultiByteBinaryOrTextStream methodsFor: 'accessing' stamp: 'yo 11/11/2002 13:25'!
isBinary
	^ isBinary! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 11/11/2002 16:39'!
next

	| n |
	n := self converter nextFromStream: self.
	n ifNil: [^ nil].
	isBinary and: [n isCharacter ifTrue: [^ n asciiValue]].
	^ n.
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'ar 4/12/2005 17:34'!
next: anInteger 

	| multiString |
	"self halt."
	self isBinary ifTrue: [^ (super next: anInteger) asByteArray].
	multiString := WideString new: anInteger.
	1 to: anInteger do: [:index |
		| character |
		(character := self next) ifNotNil: [
			multiString at: index put: character
		] ifNil: [
			multiString := multiString copyFrom: 1 to: index - 1.
			^ multiString
		]
	].
	^ multiString.
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'PeterHugossonMiller 9/3/2009 10:07'!
nextDelimited: terminator

	| out ch pos |
	out := (String new: 1000) writeStream.
	self atEnd ifTrue: [^ ''].
	pos := self position.
	self next = terminator ifFalse: [
		"absorb initial terminator"
		self position: pos.
	].
	[(ch := self next) == nil] whileFalse: [
		(ch = terminator) ifTrue: [
			self peek = terminator ifTrue: [
				self next.  "skip doubled terminator"
			] ifFalse: [
				^ out contents  "terminator is not doubled; we're done!!"
			].
		].
		out nextPut: ch.
	].
	^ out contents.
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 11/11/2002 13:24'!
nextMatchAll: aColl

    | save |
    save := converter saveStateOf: self.
    aColl do: [:each |
       (self next) = each ifFalse: [
            converter restoreStateOf: self with: save.
            ^ false.
		].
	].
    ^ true.
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'LukasRenggli 5/8/2010 18:43'!
nextPut: aCharacter
	^ aCharacter isInteger 
		ifTrue: [ super nextPut: aCharacter asCharacter ]
		ifFalse: [ self converter nextPut: aCharacter toStream: self ]! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'LukasRenggli 5/8/2010 18:43'!
nextPutAll: aCollection
	^ self isBinary
		ifTrue: [ super nextPutAll: aCollection ]
		ifFalse: [ aCollection do: [ :each | self nextPut: each ] ]! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 11/14/2002 13:54'!
padToEndWith: aChar
	"We don't have pages, so we are at the end, and don't need to pad."! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 12/25/2003 16:04'!
peek
	"Answer what would be returned if the message next were sent to the receiver. If the receiver is at the end, answer nil.  "

	| next pos |
	self atEnd ifTrue: [^ nil].
	pos := self position.
	next := self next.
	self position: pos.
	^ next.

! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 11/11/2002 13:25'!
peekFor: item 

	| next state |
	"self atEnd ifTrue: [^ false]. -- SFStream will give nil"
	state := converter saveStateOf: self.
	(next := self next) == nil ifTrue: [^ false].
	item = next ifTrue: [^ true].
	converter restoreStateOf: self with: state.
	^ false.
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'nk 7/29/2004 12:02'!
reset

	super reset.
	isBinary ifNil: [isBinary := false].
	collection class == ByteArray ifTrue: ["Store as String and convert as needed."
		collection := collection asString.
		isBinary := true].

	self converter. "ensure that we have a converter."! !

!MultiByteBinaryOrTextStream methodsFor: 'filein/out' stamp: 'StephaneDucasse 2/3/2010 22:18'!
setConverterForCode

	| current |
	current := converter saveStateOf: self.
	self position: 0.
	self binary.
	((self next: 3) =  #[239 187 191]) ifTrue: [
		self converter: UTF8TextConverter new
	] ifFalse: [
		self converter: MacRomanTextConverter new.
	].
	converter restoreStateOf: self with: current.
	self text.
! !

!MultiByteBinaryOrTextStream methodsFor: 'filein/out' stamp: 'MarcusDenker 2/14/2010 09:30'!
setEncoderForSourceCodeNamed: streamName

	| l |
	l := streamName asLowercase.
	((l endsWith: 'cs') or: [
		(l endsWith: 'st') or: [
			(l endsWith: ('st.gz')) or: [
				(l endsWith: ('st.gz'))]]]) ifTrue: [
					self converter: MacRomanTextConverter new.
					^ self.
	].
	self converter: UTF8TextConverter new.
! !

!MultiByteBinaryOrTextStream methodsFor: 'properties-setting' stamp: 'yo 11/14/2002 13:49'!
setFileTypeToObject
	"do nothing.  We don't have a file type"! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 11/11/2002 16:17'!
skipSeparators

	[self atEnd] whileFalse: [
		self basicNext isSeparator ifFalse: [
			^ self position: self position - 1]]

! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'yo 12/25/2003 16:04'!
skipSeparatorsAndPeekNext

	"A special function to make nextChunk fast"
	| peek pos |
	[self atEnd] whileFalse: [
		pos := self position.
		(peek := self next) isSeparator ifFalse: [
			self position: pos.
			^ peek.
		].
	].
! !

!MultiByteBinaryOrTextStream methodsFor: 'accessing' stamp: 'yo 11/11/2002 16:33'!
text
	isBinary := false
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'PeterHugossonMiller 9/3/2009 10:08'!
upTo: delim 

	| out ch |
	out := (String new: 1000) writeStream.
	self atEnd ifTrue: [^ ''].
	[(ch := self next) isNil] whileFalse: [
		(ch = delim) ifTrue: [
			^ out contents  "terminator is not doubled; we're done!!"
		].
		out nextPut: ch.
	].
	^ out contents.
! !

!MultiByteBinaryOrTextStream methodsFor: 'public' stamp: 'PeterHugossonMiller 9/3/2009 10:08'!
upToEnd

	| newStream element newCollection |
	newCollection := self isBinary
				ifTrue: [ByteArray new: 100]
				ifFalse: [String new: 100].
	newStream := newCollection writeStream.
	[(element := self next) notNil]
		whileTrue: [newStream nextPut: element].
	^ newStream contents
! !

Object subclass: #CompoundTextConverterState
	instanceVariableNames: 'g0Size g1Size g0Leading g1Leading charSize streamPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!CompoundTextConverterState commentStamp: '<historical>' prior: 0!
This represents the state of CompoundTextConverter.!

!CompoundTextConverterState class methodsFor: 'instance creation' stamp: 'yo 8/19/2002 17:04'!
g0Size: g0 g1Size: g1 g0Leading: g0l g1Leading: g1l charSize: cSize streamPosition: pos

	^ (self new)
		g0Size: g0
		g1Size: g1
		g0Leading: g0l
		g1Leading: g1l
		charSize: cSize
		streamPosition: pos
	; yourself.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 21:30'!
charSize

	^ charSize
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 9/16/2002 20:41'!
charSize: s

	charSize := s.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 21:29'!
g0Leading

	^ g0Leading
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 9/16/2002 20:41'!
g0Leading: l

	g0Leading := l.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 21:29'!
g0Size

	^ g0Size
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 9/16/2002 20:41'!
g0Size: s

	g0Size := s.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 14:37'!
g0Size: g0 g1Size: g1 g0Leading: g0l g1Leading: g1l charSize: cSize streamPosition: pos

	g0Size := g0.
	g1Size := g1.
	g0Leading := g0l.
	g1Leading := g1l.
	charSize := cSize.
	streamPosition := pos.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 21:30'!
g1Leading

	^ g1Leading
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 9/16/2002 20:41'!
g1Leading: l

	g1Leading := l.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 21:29'!
g1Size

	^ g1Size
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 9/16/2002 20:41'!
g1Size: s

	g1Size := s.
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 11/4/2002 12:31'!
printOn: aStream

	aStream nextPut: $(;
		nextPutAll: g0Size printString; space;
		nextPutAll: g1Size printString; space;
		nextPutAll: g0Leading printString; space;
		nextPutAll: g1Leading printString; space;
		nextPutAll: charSize printString; space;
		nextPutAll: streamPosition printString.
	aStream nextPut: $).
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 8/23/2002 21:30'!
streamPosition

	^ streamPosition
! !

!CompoundTextConverterState methodsFor: 'accessing' stamp: 'yo 9/16/2002 20:40'!
streamPosition: pos

	streamPosition := pos.
! !

Object subclass: #TextConverter
	instanceVariableNames: 'latin1Map latin1Encodings'
	classVariableNames: ''
	poolDictionaries: 'EventSensorConstants'
	category: 'Multilingual-TextConversion'!

TextConverter class
	instanceVariableNames: 'latin1Map latin1Encodings'!

!TextConverter commentStamp: '<historical>' prior: 0!
The abstract class for all different type of text converters.  nextFromStream: and nextPut:toStream: are the public accessible methods.  If you are going to make a subclass for a stateful text conversion, you should override restoreStateOf:with: and saveStateOf: along the line of CompoundTextConverter.
!

TextConverter class
	instanceVariableNames: 'latin1Map latin1Encodings'!

TextConverter subclass: #ByteTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'EventSensorConstants'
	category: 'Multilingual-TextConversion'!

ByteTextConverter class
	instanceVariableNames: 'byteToUnicode unicodeToByte'!

!ByteTextConverter commentStamp: 'michael.rueger 1/27/2009 18:00' prior: 0!
A ByteTextConverter is the abstract class for text converters on single byte encodings.!

ByteTextConverter class
	instanceVariableNames: 'byteToUnicode unicodeToByte'!

!ByteTextConverter class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:42'!
byteToUnicodeSpec
	"Sepcify a table mapping the entries 0x80 to 0xFF to their unicode counterparts by returning a 128 element array..
	The entries 0x00 to 0x7F map to identical values so we don't need to specify them.
	See #generateByteToUnicodeSpec: for an automated way to generate these mappings."

	self subclassResponsibility! !

!ByteTextConverter class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/7/2012 21:49'!
byteToUnicodeTable
	"Return the table mapping from my byte based encoding to unicode"
	
	^ byteToUnicode! !

!ByteTextConverter class methodsFor: 'class initialization' stamp: ''!
generateByteToUnicodeSpec: url
	"Return the formatted source code for an array mapping 
	the top 128 byte to unicode values from a Unicode.org url"
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'."

	| mapping |
	mapping := self parseUnicodeOrgSpec: url.  
	^ String streamContents: [ :stream |
		stream tab; << '"'; << self name; << ' generateByteToUnicodeSpec: '; print: url; << '"'; cr; cr; tab; << '^ #('.
		128 to: 255 do: [ :each | | unicode |
			each \\ 8 = 0 ifTrue: [ stream cr; tab ].
			(unicode := mapping at: each ifAbsent: [ nil ]) isNil
				ifTrue: [ stream print: nil; space ]  
				ifFalse: [ stream << '16r' << (unicode printPaddedWith: $0 to: 4 base: 16); space ] ].
		stream nextPut: $); cr ]! !

!ByteTextConverter class methodsFor: 'class initialization' stamp: 'michael.rueger 1/27/2009 18:40'!
initialize
	"ByteTextConverter initialize"

	self allSubclassesDo: [:subclass |
		subclass initializeTables]! !

!ByteTextConverter class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:56'!
initializeTables
	"Initialize the mappings to and from Unicode."

	byteToUnicode := Array new: 128.
	unicodeToByte := Dictionary new.
	"Mind the offset because first 128 characters are not stored into byteToUnicodeSpec"
	"Some entries are nil, make them pass-through for compatibility, for now"
	self byteToUnicodeSpec keysAndValuesDo: [ :index :unicode | | unicodeValue |
		unicodeValue := unicode ifNil: [ 127 + index ] ifNotNil: [ unicode ].
		byteToUnicode at: index put: (Character value: unicodeValue).
		unicodeToByte at: unicodeValue put: (127 + index) asCharacter ]! !

!ByteTextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 1/30/2009 11:01'!
languageEnvironment
	self subclassResponsibility! !

!ByteTextConverter class methodsFor: 'class initialization' stamp: ''!
parseUnicodeOrgSpec: url
	"Parse and return a mapping from byte to unicode values from url."
	"self parseUnicodeOrgSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'."
	
	| mapping |
	mapping := Dictionary new: 256.
	(ZnClient new systemPolicy; beOneShot; get: url; contents) linesDo: [ :each |
		(each isEmpty or: [ each beginsWith: '#' ])
			ifFalse: [ | tokens hexReader |
				hexReader := [ :string | Integer readFrom: (string readStream skip: 2; yourself) base: 16 ].
				tokens := each findTokens: String tab. 
				(tokens last = '<control>' or: [ tokens last = '#UNDEFINED' ]) ifFalse: [
					mapping
						at: (hexReader value: tokens first)
						put: (hexReader value: tokens second) ] ] ].
	^ mapping ! !

!ByteTextConverter class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/7/2012 21:50'!
unicodeToByteTable
	"Return the table mapping from unicode to my byte based encoding"
	
	^ unicodeToByte! !

!ByteTextConverter methodsFor: 'conversion' stamp: 'michael.rueger 2/5/2009 14:11'!
byteToUnicode: char
	"Map from my byte based encoding to unicode.
	Due to the leading char encoding this is not strictly true, but hopefully at some point we can get rid of the leading char overhead."
	| value |
	value := char charCode.

	value < 128
		ifTrue: [^ char].
	value > 255
		ifTrue: [^ char].
	^self class byteToUnicodeTable at: (value - 128 + 1)! !

!ByteTextConverter methodsFor: 'conversion' stamp: 'michael.rueger 2/5/2009 14:11'!
nextFromStream: aStream
	"Read the next byte (we are only dealing with byte based encodings here) character from aStream and return the result converted to unicode."

	| byteOrChar |
	byteOrChar := aStream basicNext.
	aStream isBinary
		ifTrue: [^byteOrChar].
	^byteOrChar
		ifNotNil: [self byteToUnicode: byteOrChar]! !

!ByteTextConverter methodsFor: 'conversion' stamp: 'michael.rueger 2/5/2009 14:10'!
nextPut: unicodeCharacter toStream: aStream
	"Write the unicode character to aStream."

	aStream isBinary
		ifTrue: [aStream basicNextPut: unicodeCharacter charCode]
		ifFalse: [aStream basicNextPut: (self unicodeToByte: unicodeCharacter)]! !

!ByteTextConverter methodsFor: 'conversion' stamp: 'michael.rueger 2/5/2009 14:10'!
unicodeToByte: unicodeChar

	^unicodeChar charCode < 128
		ifTrue: [unicodeChar]
		ifFalse: [self class unicodeToByteTable at: unicodeChar charCode ifAbsent: [0 asCharacter]]! !

ByteTextConverter subclass: #CP1250TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!CP1250TextConverter commentStamp: '<historical>' prior: 0!
Text converter for CP1250.  Windows code page used in Eastern Europe.!

!CP1250TextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:36'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1250.TXT'"

	^ #(
	16r20AC nil 16r201A nil 16r201E 16r2026 16r2020 16r2021 
	nil 16r2030 16r0160 16r2039 16r015A 16r0164 16r017D 16r0179 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil 16r2122 16r0161 16r203A 16r015B 16r0165 16r017E 16r017A 
	16r00A0 16r02C7 16r02D8 16r0141 16r00A4 16r0104 16r00A6 16r00A7 
	16r00A8 16r00A9 16r015E 16r00AB 16r00AC 16r00AD 16r00AE 16r017B 
	16r00B0 16r00B1 16r02DB 16r0142 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r0105 16r015F 16r00BB 16r013D 16r02DD 16r013E 16r017C 
	16r0154 16r00C1 16r00C2 16r0102 16r00C4 16r0139 16r0106 16r00C7 
	16r010C 16r00C9 16r0118 16r00CB 16r011A 16r00CD 16r00CE 16r010E 
	16r0110 16r0143 16r0147 16r00D3 16r00D4 16r0150 16r00D6 16r00D7 
	16r0158 16r016E 16r00DA 16r0170 16r00DC 16r00DD 16r0162 16r00DF 
	16r0155 16r00E1 16r00E2 16r0103 16r00E4 16r013A 16r0107 16r00E7 
	16r010D 16r00E9 16r0119 16r00EB 16r011B 16r00ED 16r00EE 16r010F 
	16r0111 16r0144 16r0148 16r00F3 16r00F4 16r0151 16r00F6 16r00F7 
	16r0159 16r016F 16r00FA 16r0171 16r00FC 16r00FD 16r0163 16r02D9 )! !

!CP1250TextConverter class methodsFor: 'accessing' stamp: 'pk 1/19/2005 14:35'!
encodingNames 

	^ #('cp-1250') copy
! !

!CP1250TextConverter class methodsFor: 'as yet unclassified' stamp: ''!
initialize
	self initializeTables! !

!CP1250TextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 1/30/2009 11:02'!
languageEnvironment
	^Latin2Environment! !

ByteTextConverter subclass: #CP1252TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!CP1252TextConverter commentStamp: '<historical>' prior: 0!
Text converter for CP1252.  Windows code page used in Western Europe.!

!CP1252TextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:37'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1252.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	16r02C6 16r2030 16r0160 16r2039 16r0152 nil 16r017D nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	16r02DC 16r2122 16r0161 16r203A 16r0153 nil 16r017E 16r0178 
	16r00A0 16r00A1 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00BA 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r00D0 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r00F0 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r00FF )! !

!CP1252TextConverter class methodsFor: 'accessing' stamp: 'pmm 8/16/2010 10:59'!
encodingNames 

	^ #('cp-1252') copy
! !

!CP1252TextConverter class methodsFor: 'initialization' stamp: 'pmm 8/16/2010 10:58'!
initialize
	self initializeTables! !

!CP1252TextConverter class methodsFor: 'accessing' stamp: 'pmm 8/16/2010 10:30'!
languageEnvironment
	^Latin9Environment! !

ByteTextConverter subclass: #CP1253TextConverter
	instanceVariableNames: ''
	classVariableNames: 'FromTable'
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!CP1253TextConverter commentStamp: '<historical>' prior: 0!
Text converter for CP1253.  Windows code page used for Greek.!

!CP1253TextConverter class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/7/2012 21:37'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1253.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	nil 16r2030 nil 16r2039 nil nil nil nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil 16r2122 nil 16r203A nil nil nil nil 
	16r00A0 16r0385 16r0386 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 nil 16r00AB 16r00AC 16r00AD 16r00AE 16r2015 
	16r00B0 16r00B1 16r00B2 16r00B3 16r0384 16r00B5 16r00B6 16r00B7 
	16r0388 16r0389 16r038A 16r00BB 16r038C 16r00BD 16r038E 16r038F 
	16r0390 16r0391 16r0392 16r0393 16r0394 16r0395 16r0396 16r0397 
	16r0398 16r0399 16r039A 16r039B 16r039C 16r039D 16r039E 16r039F 
	16r03A0 16r03A1 nil 16r03A3 16r03A4 16r03A5 16r03A6 16r03A7 
	16r03A8 16r03A9 16r03AA 16r03AB 16r03AC 16r03AD 16r03AE 16r03AF 
	16r03B0 16r03B1 16r03B2 16r03B3 16r03B4 16r03B5 16r03B6 16r03B7 
	16r03B8 16r03B9 16r03BA 16r03BB 16r03BC 16r03BD 16r03BE 16r03BF 
	16r03C0 16r03C1 16r03C2 16r03C3 16r03C4 16r03C5 16r03C6 16r03C7 
	16r03C8 16r03C9 16r03CA 16r03CB 16r03CC 16r03CD 16r03CE nil )! !

!CP1253TextConverter class methodsFor: 'accessing' stamp: 'yo 2/19/2004 10:11'!
encodingNames 

	^ #('cp-1253') copy
! !

!CP1253TextConverter class methodsFor: 'as yet unclassified' stamp: ''!
initialize
	self initializeTables! !

!CP1253TextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 2/2/2009 18:49'!
languageEnvironment
	^GreekEnvironment! !

ByteTextConverter subclass: #ISO885915TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!ISO885915TextConverter commentStamp: '<historical>' prior: 0!
Text converter for ISO 8859-15.  An international encoding used in Western Europe, similar to ISO 8859-1 but adds the Euro symbol and drops others like one quarter.!

!ISO885915TextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:38'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-15.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r00A1 16r00A2 16r00A3 16r20AC 16r00A5 16r0160 16r00A7 
	16r0161 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r017D 16r00B5 16r00B6 16r00B7 
	16r017E 16r00B9 16r00BA 16r00BB 16r0152 16r0153 16r0178 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r00D0 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r00F0 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r00FF )! !

!ISO885915TextConverter class methodsFor: 'accessing' stamp: 'pmm 8/16/2010 10:30'!
encodingNames 

	^ #('iso-8859-15') copy
! !

!ISO885915TextConverter class methodsFor: 'initialization' stamp: 'pmm 8/16/2010 10:58'!
initialize
	self initializeTables! !

!ISO885915TextConverter class methodsFor: 'accessing' stamp: 'pmm 8/16/2010 10:30'!
languageEnvironment
	^Latin9Environment! !

ByteTextConverter subclass: #ISO88592TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!ISO88592TextConverter commentStamp: '<historical>' prior: 0!
Text converter for ISO 8859-2.  An international encoding used in Eastern Europe.!

!ISO88592TextConverter class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/7/2012 21:39'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0104 16r02D8 16r0141 16r00A4 16r013D 16r015A 16r00A7 
	16r00A8 16r0160 16r015E 16r0164 16r0179 16r00AD 16r017D 16r017B 
	16r00B0 16r0105 16r02DB 16r0142 16r00B4 16r013E 16r015B 16r02C7 
	16r00B8 16r0161 16r015F 16r0165 16r017A 16r02DD 16r017E 16r017C 
	16r0154 16r00C1 16r00C2 16r0102 16r00C4 16r0139 16r0106 16r00C7 
	16r010C 16r00C9 16r0118 16r00CB 16r011A 16r00CD 16r00CE 16r010E 
	16r0110 16r0143 16r0147 16r00D3 16r00D4 16r0150 16r00D6 16r00D7 
	16r0158 16r016E 16r00DA 16r0170 16r00DC 16r00DD 16r0162 16r00DF 
	16r0155 16r00E1 16r00E2 16r0103 16r00E4 16r013A 16r0107 16r00E7 
	16r010D 16r00E9 16r0119 16r00EB 16r011B 16r00ED 16r00EE 16r010F 
	16r0111 16r0144 16r0148 16r00F3 16r00F4 16r0151 16r00F6 16r00F7 
	16r0159 16r016F 16r00FA 16r0171 16r00FC 16r00FD 16r0163 16r02D9 )! !

!ISO88592TextConverter class methodsFor: 'utilities' stamp: 'yo 1/18/2005 09:17'!
encodingNames 

	^ #('iso-8859-2') copy
! !

!ISO88592TextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:46'!
initialize
	self initializeTables! !

!ISO88592TextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 2/2/2009 18:48'!
languageEnvironment
	^Latin2Environment! !

ByteTextConverter subclass: #ISO88597TextConverter
	instanceVariableNames: ''
	classVariableNames: 'FromTable'
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!ISO88597TextConverter commentStamp: '<historical>' prior: 0!
Text converter for ISO 8859-7.  An international encoding used for Greek.!

!ISO88597TextConverter class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/7/2012 21:39'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-7.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r2018 16r2019 16r00A3 16r20AC 16r20AF 16r00A6 16r00A7 
	16r00A8 16r00A9 16r037A 16r00AB 16r00AC 16r00AD nil 16r2015 
	16r00B0 16r00B1 16r00B2 16r00B3 16r0384 16r0385 16r0386 16r00B7 
	16r0388 16r0389 16r038A 16r00BB 16r038C 16r00BD 16r038E 16r038F 
	16r0390 16r0391 16r0392 16r0393 16r0394 16r0395 16r0396 16r0397 
	16r0398 16r0399 16r039A 16r039B 16r039C 16r039D 16r039E 16r039F 
	16r03A0 16r03A1 nil 16r03A3 16r03A4 16r03A5 16r03A6 16r03A7 
	16r03A8 16r03A9 16r03AA 16r03AB 16r03AC 16r03AD 16r03AE 16r03AF 
	16r03B0 16r03B1 16r03B2 16r03B3 16r03B4 16r03B5 16r03B6 16r03B7 
	16r03B8 16r03B9 16r03BA 16r03BB 16r03BC 16r03BD 16r03BE 16r03BF 
	16r03C0 16r03C1 16r03C2 16r03C3 16r03C4 16r03C5 16r03C6 16r03C7 
	16r03C8 16r03C9 16r03CA 16r03CB 16r03CC 16r03CD 16r03CE nil )! !

!ISO88597TextConverter class methodsFor: 'utilities' stamp: 'yo 2/10/2004 06:32'!
encodingNames 

	^ #('iso-8859-7' 'greek-iso-8859-8bit') copy
! !

!ISO88597TextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:46'!
initialize
	self initializeTables! !

!ISO88597TextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 2/2/2009 18:49'!
languageEnvironment
	^GreekEnvironment! !

ByteTextConverter subclass: #KOI8RTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!KOI8RTextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:40'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MISC/KOI8-R.TXT'"

	^ #(
	16r2500 16r2502 16r250C 16r2510 16r2514 16r2518 16r251C 16r2524 
	16r252C 16r2534 16r253C 16r2580 16r2584 16r2588 16r258C 16r2590 
	16r2591 16r2592 16r2593 16r2320 16r25A0 16r2219 16r221A 16r2248 
	16r2264 16r2265 16r00A0 16r2321 16r00B0 16r00B2 16r00B7 16r00F7 
	16r2550 16r2551 16r2552 16r0451 16r2553 16r2554 16r2555 16r2556 
	16r2557 16r2558 16r2559 16r255A 16r255B 16r255C 16r255D 16r255E 
	16r255F 16r2560 16r2561 16r0401 16r2562 16r2563 16r2564 16r2565 
	16r2566 16r2567 16r2568 16r2569 16r256A 16r256B 16r256C 16r00A9 
	16r044E 16r0430 16r0431 16r0446 16r0434 16r0435 16r0444 16r0433 
	16r0445 16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 
	16r043F 16r044F 16r0440 16r0441 16r0442 16r0443 16r0436 16r0432 
	16r044C 16r044B 16r0437 16r0448 16r044D 16r0449 16r0447 16r044A 
	16r042E 16r0410 16r0411 16r0426 16r0414 16r0415 16r0424 16r0413 
	16r0425 16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 
	16r041F 16r042F 16r0420 16r0421 16r0422 16r0423 16r0416 16r0412 
	16r042C 16r042B 16r0417 16r0428 16r042D 16r0429 16r0427 16r042A )! !

!KOI8RTextConverter class methodsFor: 'accessing' stamp: 'yo 12/11/2007 10:59'!
encodingNames 

	^ #('koi8-r') copy
! !

!KOI8RTextConverter class methodsFor: 'as yet unclassified' stamp: ''!
initialize
	self initializeTables! !

!KOI8RTextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 2/2/2009 18:53'!
languageEnvironment
	^RussianEnvironment! !

ByteTextConverter subclass: #Latin1TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!Latin1TextConverter commentStamp: '<historical>' prior: 0!
Text converter for ISO 8859-1.  An international encoding used in Western Europe.!

!Latin1TextConverter class methodsFor: 'accessing' stamp: ''!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-1.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r00A1 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00BA 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r00D0 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r00F0 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r00FF )! !

!Latin1TextConverter class methodsFor: 'accessing' stamp: 'tbn 1/17/2011 13:04'!
encodingNames 

	^ #('latin-1' 'latin1' 'iso-8859-1') copy.
! !

!Latin1TextConverter class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 23:17'!
initializeTables
	"Overwritten to be a noop for Latin1,
	we don't use #byteToUnicodeSpec although we should"! !

!Latin1TextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 2/2/2009 19:31'!
languageEnvironment
	^Latin1Environment! !

!Latin1TextConverter methodsFor: 'conversion' stamp: 'michael.rueger 2/5/2009 14:20'!
byteToUnicode: char

	^char! !

!Latin1TextConverter methodsFor: 'conversion' stamp: 'michael.rueger 2/5/2009 14:20'!
unicodeToByte: unicodeChar

	^unicodeChar charCode < 256
		ifTrue: [unicodeChar]
		ifFalse: [0 asCharacter]! !

ByteTextConverter subclass: #MacRomanTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!MacRomanTextConverter commentStamp: '<historical>' prior: 0!
Text converter for Mac Roman.  An encoding used for the languages originated from Western Europe area.!

!MacRomanTextConverter class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 3/7/2012 21:41'!
byteToUnicodeSpec
	"ByteTextConverter generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/APPLE/ROMAN.TXT'"

	^ #(
	16r00C4 16r00C5 16r00C7 16r00C9 16r00D1 16r00D6 16r00DC 16r00E1 
	16r00E0 16r00E2 16r00E4 16r00E3 16r00E5 16r00E7 16r00E9 16r00E8 
	16r00EA 16r00EB 16r00ED 16r00EC 16r00EE 16r00EF 16r00F1 16r00F3 
	16r00F2 16r00F4 16r00F6 16r00F5 16r00FA 16r00F9 16r00FB 16r00FC 
	16r2020 16r00B0 16r00A2 16r00A3 16r00A7 16r2022 16r00B6 16r00DF 
	16r00AE 16r00A9 16r2122 16r00B4 16r00A8 16r2260 16r00C6 16r00D8 
	16r221E 16r00B1 16r2264 16r2265 16r00A5 16r00B5 16r2202 16r2211 
	16r220F 16r03C0 16r222B 16r00AA 16r00BA 16r03A9 16r00E6 16r00F8 
	16r00BF 16r00A1 16r00AC 16r221A 16r0192 16r2248 16r2206 16r00AB 
	16r00BB 16r2026 16r00A0 16r00C0 16r00C3 16r00D5 16r0152 16r0153 
	16r2013 16r2014 16r201C 16r201D 16r2018 16r2019 16r00F7 16r25CA 
	16r00FF 16r0178 16r2044 16r20AC 16r2039 16r203A 16rFB01 16rFB02 
	16r2021 16r00B7 16r201A 16r201E 16r2030 16r00C2 16r00CA 16r00C1 
	16r00CB 16r00C8 16r00CD 16r00CE 16r00CF 16r00CC 16r00D3 16r00D4 
	16rF8FF 16r00D2 16r00DA 16r00DB 16r00D9 16r0131 16r02C6 16r02DC 
	16r00AF 16r02D8 16r02D9 16r02DA 16r00B8 16r02DD 16r02DB 16r02C7 )! !

!MacRomanTextConverter class methodsFor: 'accessing' stamp: 'yo 8/4/2003 12:33'!
encodingNames 

	^ #('mac-roman' ) copy
! !

!MacRomanTextConverter class methodsFor: 'as yet unclassified' stamp: ''!
initialize
	self initializeTables! !

!MacRomanTextConverter class methodsFor: 'accessing' stamp: 'michael.rueger 2/2/2009 19:31'!
languageEnvironment
	^Latin1Environment! !

TextConverter subclass: #CompoundTextConverter
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!CompoundTextConverter commentStamp: '<historical>' prior: 0!
Text converter for X Compound Text.!

!CompoundTextConverter class methodsFor: 'utilities' stamp: 'yo 10/24/2002 14:16'!
encodingNames

	^ #('iso-2022-jp' 'x-ctext') copy
! !

!CompoundTextConverter methodsFor: 'friend' stamp: 'yo 8/18/2003 17:50'!
emitSequenceToResetStateIfNeededOn: aStream

	Latin1 emitSequenceToResetStateIfNeededOn: aStream forState: state.
! !

!CompoundTextConverter methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 09:49'!
initialize
	super initialize.
	state := CompoundTextConverterState 
		g0Size: 1 g1Size: 1 g0Leading: 0 g1Leading: 0 charSize: 1 streamPosition: 0.

"	unused
	acceptingEncodings := #(ascii iso88591 jisx0208 gb2312 ksc5601 ksx1001 ) copy."
! !

!CompoundTextConverter methodsFor: 'conversion' stamp: 'ar 4/9/2005 22:25'!
nextFromStream: aStream 

	| character character2 size leadingChar offset result |
	aStream isBinary ifTrue: [^ aStream basicNext].

	character := aStream basicNext.
	character ifNil: [^ nil].
	character == Character escape ifTrue: [
		self parseShiftSeqFromStream: aStream.
		character := aStream basicNext.
		character ifNil: [^ nil]].
	character asciiValue < 128 ifTrue: [
		size := state g0Size.
		leadingChar := state g0Leading.
		offset := 16r21.
	] ifFalse: [
		size :=state g1Size.
		leadingChar := state g1Leading.
		offset := 16rA1.
	].
	size = 1 ifTrue: [
		leadingChar = 0
			ifTrue: [^ character]
			ifFalse: [^ Character leadingChar: leadingChar code: character asciiValue]
	].
	size = 2 ifTrue: [
		character2 := aStream basicNext.
		character2 ifNil: [^ nil. "self errorMalformedInput"].
		character := character asciiValue - offset.
		character2 := character2 asciiValue - offset.
		result := Character leadingChar: leadingChar code: character * 94 + character2.
		^ result asUnicodeChar.
		"^ self toUnicode: result"
	].
	self error: 'unsupported encoding'.
! !

!CompoundTextConverter methodsFor: 'conversion' stamp: 'MarcusDenker 3/28/2011 22:15'!
nextPut: aCharacter toStream: aStream

	| ascii leadingChar class |
	aStream isBinary ifTrue: [^aCharacter storeBinaryOn: aStream].
	class := (EncodedCharSet charsetAt: aCharacter leadingChar) traditionalCharsetClass.
	ascii := (class charFromUnicode: aCharacter asUnicode) charCode.
	leadingChar := class leadingChar.
	self nextPutValue: ascii toStream: aStream withShiftSequenceIfNeededForLeadingChar: leadingChar.
! !

!CompoundTextConverter methodsFor: 'private' stamp: 'yo 11/4/2002 14:36'!
nextPutValue: ascii toStream: aStream withShiftSequenceIfNeededForLeadingChar: leadingChar

	| charset |
	charset := EncodedCharSet charsetAt: leadingChar.
	charset ifNotNil: [
		charset nextPutValue: ascii toStream: aStream withShiftSequenceIfNeededForTextConverterState: state.
	] ifNil: [
		"..."
	].
! !

!CompoundTextConverter methodsFor: 'private' stamp: 'marcus.denker 9/14/2008 21:14'!
parseShiftSeqFromStream: aStream

	| c set target id |
	c := aStream basicNext.
	c = $$ ifTrue: [
		set := #multibyte.
		c := aStream basicNext.
		c = $( ifTrue: [target := 1].
		c = $) ifTrue: [target := 2].
		target ifNil: [target := 1. id := c]
			ifNotNil: [id := aStream basicNext].
	] ifFalse: [
		c = $( ifTrue: [target := 1. set := #nintyfour].
		c = $) ifTrue: [target := 2. set := #nintyfour].
		c = $- ifTrue: [target := 2. set := #nintysix].
		id := aStream basicNext.
	].
	(set = #multibyte and: [id = $B]) ifTrue: [
		state charSize: 2.
		target = 1 ifTrue: [
			state g0Size: 2.
			state g0Leading: 1.
		] ifFalse: [
			state g1Size: 2.
			state g1Leading: 1.
		].
		^ self
	].
	(set = #multibyte and: [id = $A]) ifTrue: [
		state charSize: 2.
		target = 1 ifTrue: [
			state g0Size: 2.
			state g0Leading: 2.
		] ifFalse: [
			state g1Size: 2.
			state g1Leading: 2.
		].
		^ self
	].

	(set = #nintyfour and: [id = $B or: [id = $J]]) ifTrue: [
		state charSize: 1.
		state g0Size: 1.
		state g0Leading: 0.
		^ self
	].
	(set = #nintysix and: [id = $A]) ifTrue: [
		state charSize: 1.
		state g1Size: 1.
		state g1Leading: 0.
		^ self
	].! !

!CompoundTextConverter methodsFor: 'friend' stamp: 'yo 11/4/2002 12:33'!
restoreStateOf: aStream with: aConverterState

	state := aConverterState copy.
	aStream position: state streamPosition.
! !

!CompoundTextConverter methodsFor: 'friend' stamp: 'pmm 3/13/2010 11:22'!
saveStateOf: aStream

	| inst |
	inst :=  state shallowCopy.
	inst streamPosition: aStream position.
	^ inst.
! !

TextConverter subclass: #EUCTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!EUCTextConverter commentStamp: '<historical>' prior: 0!
Text converter for Extended Unix Character.  This is an abstract class.  The CJK variations are implemented as subclasses.!

EUCTextConverter subclass: #CNGBTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!CNGBTextConverter commentStamp: '<historical>' prior: 0!
Text converter for Simplified Chinese variation of EUC.  (Even though the name doesn't look so, it is what it is.)!

!CNGBTextConverter class methodsFor: 'utilities' stamp: 'yo 10/23/2002 14:42'!
encodingNames 

	^ #('gb2312' ) copy
! !

!CNGBTextConverter methodsFor: 'private' stamp: 'yo 3/17/2004 00:41'!
languageEnvironment

	^ SimplifiedChineseEnvironment.
! !

!CNGBTextConverter methodsFor: 'private' stamp: 'yo 10/23/2002 14:42'!
leadingChar

	^ GB2312 leadingChar
! !

EUCTextConverter subclass: #EUCJPTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!EUCJPTextConverter commentStamp: '<historical>' prior: 0!
Text converter for Japanese variation of EUC.!

!EUCJPTextConverter class methodsFor: 'utilities' stamp: 'yo 12/19/2003 22:00'!
encodingNames 

	^ #('euc-jp' 'eucjp') copy
! !

!EUCJPTextConverter methodsFor: 'private' stamp: 'yo 3/17/2004 00:41'!
languageEnvironment

	^ JapaneseEnvironment.
! !

!EUCJPTextConverter methodsFor: 'private' stamp: 'yo 10/23/2002 10:09'!
leadingChar

	^ JISX0208 leadingChar
! !

EUCTextConverter subclass: #EUCKRTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!EUCKRTextConverter commentStamp: '<historical>' prior: 0!
Text converter for Korean variation of EUC.!

!EUCKRTextConverter class methodsFor: 'utilities' stamp: 'yo 2/17/2004 18:45'!
encodingNames 

	^ #('euc-kr' 'ks-c-5601-1987' 'euckr') copy
! !

!EUCKRTextConverter methodsFor: 'private' stamp: 'yo 3/17/2004 00:41'!
languageEnvironment

	^ KoreanEnvironment.
! !

!EUCKRTextConverter methodsFor: 'private' stamp: 'yo 10/23/2002 15:19'!
leadingChar

	^ KSX1001 leadingChar
! !

!EUCTextConverter methodsFor: 'private' stamp: 'yo 3/17/2004 00:40'!
languageEnvironment

	self subclassResponsibility
! !

!EUCTextConverter methodsFor: 'private' stamp: 'yo 10/23/2002 10:09'!
leadingChar

	^ self subclassResponsibility
! !

!EUCTextConverter methodsFor: 'conversion' stamp: 'nice 3/16/2010 20:22'!
nextFromStream: aStream

	| character1 character2 offset value1 value2 nonUnicodeChar |
	aStream isBinary ifTrue: [^ aStream basicNext].
	(character1 := aStream basicNext) ifNil: [^ nil].
	character1 asciiValue <= 127 ifTrue: [^ character1].
	(character2 := aStream basicNext) ifNil: [^ nil].
	offset := 16rA1.
	value1 := character1 asciiValue - offset.
	value2 := character2 asciiValue - offset.
	(value1 < 0 or: [value1 > 93]) ifTrue: [^ nil].
	(value2 < 0 or: [value2 > 93]) ifTrue: [^ nil].

	nonUnicodeChar := Character leadingChar: self leadingChar code: value1 * 94 + value2.
	^ Character leadingChar: self languageEnvironment leadingChar code: nonUnicodeChar asUnicode.
! !

!EUCTextConverter methodsFor: 'conversion' stamp: 'MarcusDenker 3/28/2011 22:15'!
nextPut: aCharacter toStream: aStream 
	| value leadingChar nonUnicodeChar value1 value2 |
	aStream isBinary ifTrue: [^aCharacter storeBinaryOn: aStream].
	value := aCharacter charCode.
	leadingChar := aCharacter leadingChar.
	(leadingChar = 0 and: [value < 128]) ifTrue: [
		aStream basicNextPut: (Character value: value).
		^ aStream
	].

	(128 <= value and: [value < 256]) ifTrue: [^ aStream].
	nonUnicodeChar := self nonUnicodeClass charFromUnicode: value.
	nonUnicodeChar ifNotNil: [
		value := nonUnicodeChar charCode.
		value1 := value // 94 + 161.
		value2 := value \\ 94 + 161.
		aStream basicNextPut: (Character value: value1).
		aStream basicNextPut: (Character value: value2).
		^ aStream
	]
! !

!EUCTextConverter methodsFor: 'private' stamp: 'yo 10/4/2003 15:48'!
nonUnicodeClass

	^ (EncodedCharSet charsetAt: self leadingChar).
! !

TextConverter subclass: #ShiftJISTextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!ShiftJISTextConverter commentStamp: '<historical>' prior: 0!
Text converter for Shift-JIS.  Mac and Windows in Japanese mode use this encoding.!

!ShiftJISTextConverter class methodsFor: 'utilities' stamp: 'yo 12/25/2003 21:33'!
encodingNames 

	^ #('shift-jis' 'shift_jis' 'sjis') copy
! !

!ShiftJISTextConverter methodsFor: 'private' stamp: 'ar 4/9/2005 22:31'!
katakanaValue: code

	^ Character leadingChar: JapaneseEnvironment leadingChar code: (#(
	16rFFFD 16rFF61 16rFF62 16rFF63 16rFF64 16rFF65 16rFF66 16rFF67
	16rFF68 16rFF69 16rFF6A 16rFF6B 16rFF6C 16rFF6D 16rFF6E 16rFF6F
	16rFF70 16rFF71 16rFF72 16rFF73 16rFF74 16rFF75 16rFF76 16rFF77
	16rFF78 16rFF79 16rFF7A 16rFF7B 16rFF7C 16rFF7D 16rFF7E 16rFF7F
	16rFF80 16rFF81 16rFF82 16rFF83 16rFF84 16rFF85 16rFF86 16rFF87
	16rFF88 16rFF89 16rFF8A 16rFF8B 16rFF8C 16rFF8D 16rFF8E 16rFF8F
	16rFF90 16rFF91 16rFF92 16rFF93 16rFF94 16rFF95 16rFF96 16rFF97
	16rFF98 16rFF99 16rFF9A 16rFF9B 16rFF9C 16rFF9D 16rFF9E 16rFF9F
) at: (code - 16r9F)).
! !

!ShiftJISTextConverter methodsFor: 'friend' stamp: 'yo 10/23/2002 15:28'!
leadingChar

	^ JISX0208 leadingChar
! !

!ShiftJISTextConverter methodsFor: 'conversion' stamp: 'GabrielOmarCotelli 5/25/2010 21:22'!
nextFromStream: aStream
	| character1 character2 value1 value2 char1Value result |
	aStream isBinary
		ifTrue: [ ^ aStream basicNext ].
	character1 := aStream basicNext.
	character1 isNil
		ifTrue: [ ^ nil ].
	char1Value := character1 asciiValue.
	char1Value < 16r81
		ifTrue: [ ^ character1 ].
	(char1Value > 16rA0 and: [ char1Value < 16rE0 ])
		ifTrue: [ ^ self katakanaValue: char1Value ].
	character2 := aStream basicNext.
	character2 isNil
		ifTrue: [ ^ nil	"self errorMalformedInput" ].
	value1 := character1 asciiValue.
	character1 asciiValue >= 224
		ifTrue: [ value1 := value1 - 64 ].
	value1 := value1 - 129 bitShift: 1.
	value2 := character2 asciiValue.
	character2 asciiValue >= 128
		ifTrue: [ value2 := value2 - 1 ].
	character2 asciiValue >= 158
		ifTrue: [ 
			value1 := value1 + 1.
			value2 := value2 - 158 ]
		ifFalse: [ value2 := value2 - 64 ].
	result := Character leadingChar: self leadingChar code: value1 * 94 + value2.
	^ self toUnicode: result! !

!ShiftJISTextConverter methodsFor: 'conversion' stamp: 'StephaneDucasse 5/28/2011 13:41'!
nextPut: aCharacter toStream: aStream 
	| value leadingChar aChar |
	aStream isBinary ifTrue: [^aCharacter storeBinaryOn: aStream].
	value := aCharacter charCode.
	(16rFF61 <= value and: [value <= 16rFF9F]) ifTrue: [
		aStream basicNextPut: (self sjisKatakanaFor: value).
		^ aStream].
	aChar := JISX0208 charFromUnicode: value.
	aChar ifNil: [^ aStream].
	value := aChar charCode.
	leadingChar := aChar leadingChar.
	leadingChar = 0 ifTrue: [
		aStream basicNextPut: (Character value: value).
		^ aStream.
	].
	leadingChar == self leadingChar ifTrue: [
		| upper lower | 
		upper := value // 94 + 33.
		lower := value \\ 94 + 33.
		upper \\ 2 = 1 ifTrue: [
			upper := upper + 1 / 2 + 112.
			lower := lower + 31
		] ifFalse: [
			upper := upper / 2 + 112.
			lower := lower + 125
		].
		upper >= 160 ifTrue: [upper := upper + 64].
		lower >= 127 ifTrue: [lower := lower + 1].
		aStream basicNextPut: (Character value: upper).
		aStream basicNextPut: (Character value: lower).
		^ aStream
	].
! !

!ShiftJISTextConverter methodsFor: 'private' stamp: 'yo 3/1/2004 22:05'!
sjisKatakanaFor: value

	^ Character value: (#(
		16rA0 16rA1 16rA2 16rA3 16rA4 16rA5 16rA6 16rA7
		16rA8 16rA9 16rAA 16rAB 16rAC 16rAD 16rAE 16rAF
		16rB0 16rB1 16rB2 16rB3 16rB4 16rB5 16rB6 16rB7
		16rB8 16rB9 16rBA 16rBB 16rBC 16rBD 16rBE 16rBF
		16rC0 16rC1 16rC2 16rC3 16rC4 16rC5 16rC6 16rC7
		16rC8 16rC9 16rCA 16rCB 16rCC 16rCD 16rCE 16rCF
		16rD0 16rD1 16rD2 16rD3 16rD4 16rD5 16rD6 16rD7
		16rD8 16rD9 16rDA 16rDB 16rDC 16rDD 16rDE 16rDF
	) at: value - 16rFF5F).

! !

!ShiftJISTextConverter methodsFor: 'private' stamp: 'ar 4/9/2005 22:31'!
toUnicode: aChar

	^ Character leadingChar: JapaneseEnvironment leadingChar code: aChar asUnicode.
! !

!TextConverter class methodsFor: 'utilities' stamp: 'yo 7/5/2004 19:41'!
allEncodingNames
	"TextConverter allEncodingNames"
	| encodingNames |
	encodingNames := Set new.
	self allSubclasses
		do: [:each | 
			| names | 
			names := each encodingNames.
			names notEmpty
				ifTrue: [encodingNames add: names first asSymbol]].
	^encodingNames! !

!TextConverter class methodsFor: 'instance creation' stamp: 'yo 12/28/2003 00:54'!
default

	^ UTF8TextConverter new.
! !

!TextConverter class methodsFor: 'instance creation' stamp: 'yo 7/25/2003 14:08'!
defaultConverterClassForEncoding: encodingName
	"TextConverter defaultConverterClassForEncoding: 'shift-jis'"

	^ self allSubclasses
		detect: [:class | class encodingNames includes: encodingName]
		ifNone: []
! !

!TextConverter class methodsFor: 'instance creation' stamp: 'mir 7/20/2004 15:51'!
defaultSystemConverter

	^LanguageEnvironment defaultSystemConverter! !

!TextConverter class methodsFor: 'utilities' stamp: 'yo 8/19/2002 15:28'!
encodingNames 

	^ #() copy.
! !

!TextConverter class methodsFor: 'accessing' stamp: 'nice 5/10/2009 00:45'!
initializeLatin1MapAndEncodings
	"Initialize the latin1Map and latin1Encodings.
	These variables ensure that conversions from latin1 ByteString is reasonably fast"
	
	| latin1 utf8 |
	latin1Map := ByteArray new: 256.
	latin1Encodings := Array new: 256.
	0 to: 255 do:[:i|
		utf8 := (String new: 8) writeStream.
		latin1 := String with: (Character value: i).
		self new nextPut: latin1 first toStream: utf8.
		utf8 := utf8 contents.
		latin1 = utf8 ifTrue:[
			latin1Map at: i+1 put: 0. "no translation needed"
		] ifFalse:[
			latin1Map at: i+1 put: 1. "translation needed"
			latin1Encodings at: i+1 put: utf8.
		].
	].! !

!TextConverter class methodsFor: 'accessing' stamp: 'nice 5/9/2009 23:35'!
latin1Encodings
	"Answer an Array mapping latin1 characters to conversion string"

	^latin1Encodings ifNil:
		[self initializeLatin1MapAndEncodings.
		latin1Encodings]! !

!TextConverter class methodsFor: 'accessing' stamp: 'nice 5/9/2009 23:34'!
latin1Map
	"Answer a ByteArray map telling if latin1 characters needs conversion or not"

	^latin1Map ifNil:
		[self initializeLatin1MapAndEncodings.
		latin1Map]! !

!TextConverter class methodsFor: 'instance creation' stamp: 'yo 2/21/2004 04:56'!
newForEncoding: aString 
	| class encoding |
	aString ifNil: [^ Latin1TextConverter new].
	encoding := aString asLowercase.
	class := self allSubclasses
				detect: [:each | each encodingNames includes: encoding]
				ifNone: [].
	class isNil
		ifTrue: [^ nil].
	^ class new! !

!TextConverter methodsFor: 'conversion' stamp: 'michael.rueger 1/27/2009 18:12'!
convertFromSystemString: aString

	| readStream writeStream |
	readStream := aString readStream.
	writeStream := String new writeStream.

	[readStream atEnd] whileFalse: [
		writeStream nextPut: (self nextFromStream: readStream)].
	^writeStream contents
! !

!TextConverter methodsFor: 'conversion' stamp: 'HenrikSperreJohansen 1/27/2010 18:07'!
convertToSystemString: aString

	| writeStream |
	writeStream := String new writeStream.
	self nextPutAll: aString toStream: writeStream.
	self emitSequenceToResetStateIfNeededOn: writeStream.
	^writeStream contents! !

!TextConverter methodsFor: 'friend' stamp: 'yo 7/29/2003 15:51'!
emitSequenceToResetStateIfNeededOn: aStream
! !

!TextConverter methodsFor: 'initialize-release' stamp: 'HenrikSperreJohansen 2/5/2010 20:24'!
initialize
	latin1Map := self class latin1Map.
	latin1Encodings := self class latin1Encodings.! !

!TextConverter methodsFor: 'initialize-release' stamp: 'HenrikSperreJohansen 1/27/2010 18:03'!
installLineEndConvention: lineEndStringOrNil

	
	lineEndStringOrNil ifNotNil:
		[latin1Encodings := latin1Encodings copy.
		latin1Encodings at: Character cr asciiValue + 1 put: (self convertFromSystemString: lineEndStringOrNil).
		latin1Map := ByteString nonAsciiMap copy.
		latin1Map at: Character cr asciiValue + 1 put: 1]! !

!TextConverter methodsFor: 'conversion' stamp: 'yo 8/19/2002 15:27'!
nextFromStream: aStream

	self subclassResponsibility.
! !

!TextConverter methodsFor: 'conversion' stamp: 'yo 8/19/2002 15:27'!
nextPut: aCharacter toStream: aStream

	self subclassResponsibility.
! !

!TextConverter methodsFor: 'conversion' stamp: 'HenrikSperreJohansen 8/10/2010 15:55'!
nextPutAll: aString toStream: aStream
	"Handle fast conversion if ByteString"
	
	| lastIndex nextIndex |
	aString class == ByteString ifFalse: [
		aString do: [:char | self nextPut: char toStream: aStream].
		^self].
	
	aStream isBinary ifTrue: [
		aStream basicNextPutAll: aString.
		^self].
	self nextPutByteString: aString toStream: aStream.
	^self
! !

!TextConverter methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/10/2010 16:20'!
nextPutByteString: aString toStream: aStream

	"This is a reasonable default which speeds up conversion of encodings which have a partial match with latin1 encoding (for instance ascii supersets)"
	| lastIndex nextIndex |
	lastIndex := 1.
	[nextIndex := ByteString findFirstInString: aString inSet: latin1Map startingAt: lastIndex.
	nextIndex = 0] whileFalse:
		[aStream next: nextIndex-lastIndex putAll: aString startingAt: lastIndex.
		aStream basicNextPutAll: (latin1Encodings at: (aString byteAt: nextIndex)+1).
		lastIndex := nextIndex + 1].
	aStream next: aString size-lastIndex+1 putAll: aString startingAt: lastIndex.! !

!TextConverter methodsFor: 'friend' stamp: 'yo 2/21/2004 03:26'!
restoreStateOf: aStream with: aConverterState

	aStream position: aConverterState.
! !

!TextConverter methodsFor: 'friend' stamp: 'yo 2/21/2004 03:59'!
saveStateOf: aStream

	^ aStream position.
! !

TextConverter subclass: #UTF16TextConverter
	instanceVariableNames: 'useLittleEndian useByteOrderMark byteOrderMarkDone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!UTF16TextConverter commentStamp: 'HenrikSperreJohansen 8/9/2010 15:29' prior: 0!
Text converter for UTF-16.  It supports the endianness and byte order mark.

The default is to not use a BOM, make sure to set this if using the converter to write to a location where encoding metadata is not present / specified by the protocol.

This usually means you want to write one when storing strings to files on disk, but not when transmitting encoded string to a web client, nor when sending data to a database.!

!UTF16TextConverter class methodsFor: 'utilities' stamp: 'yo 2/10/2004 05:23'!
encodingNames

	^ #('utf-16' 'utf16' 'utf-16-le' 'utf-16-be') copy.
! !

!UTF16TextConverter methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/9/2010 15:19'!
charFromStream: aStream withFirst: firstValue

	^(firstValue < 16rD800 or: [firstValue > 16rDBFF])
		ifTrue:
			[Unicode value: firstValue]
		ifFalse:  "We have a surrogate pair"
			[| character1 character2 n secondValue |
			character1 := aStream basicNext ifNil: [^nil].
			character2 := aStream basicNext ifNil: [^nil].
			self useLittleEndian ifTrue: [ |tmp|
				tmp := character1.
				character1 := character2.
				character2 := tmp].
			secondValue := (character1 charCode << 8) + (character2 charCode).
			Unicode value: (firstValue - 16rD800) * 16r400 + (secondValue - 16rDC00) + 16r10000.]
! !

!UTF16TextConverter methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/9/2010 15:01'!
checkBOMOn: aStream
	"Write bom if applicable, and we are at beginning of stream"
	(self useByteOrderMark and: [byteOrderMarkDone not]) ifTrue: [
		aStream position = 0 ifTrue: [self next16BitValue: (16rFEFF) toStream: aStream].
		byteOrderMarkDone := true.
	].! !

!UTF16TextConverter methodsFor: 'initialize-release' stamp: 'HenrikSperreJohansen 8/9/2010 15:02'!
initialize
	super initialize.
	byteOrderMarkDone := false.
	useByteOrderMark := false.
	useLittleEndian := false.! !

!UTF16TextConverter methodsFor: 'private' stamp: 'yo 1/13/2004 12:59'!
next16BitValue: value toStream: aStream

	| v1 v2 |
	v1 := (value >> 8) bitAnd: 16rFF.
	v2 := value bitAnd: 16rFF.

	self useLittleEndian ifTrue: [
		aStream basicNextPut: (Character value: v2).
		aStream basicNextPut: (Character value: v1).
	] ifFalse: [
		aStream basicNextPut: (Character value: v1).
		aStream basicNextPut: (Character value: v2).
	].
! !

!UTF16TextConverter methodsFor: 'conversion' stamp: 'HenrikSperreJohansen 8/9/2010 15:12'!
nextFromStream: aStream

	| character1 character2 readBOM charValue |
	aStream isBinary ifTrue: [^ aStream basicNext].
	character1 := aStream basicNext ifNil: [^nil].
	character2 := aStream basicNext ifNil: [^nil].

	readBOM := false.
	(character1 asciiValue = 16rFF and: [character2 asciiValue = 16rFE]) ifTrue: 
		[self useLittleEndian: true.
		readBOM := true].
	(character1 asciiValue = 16rFE and: [character2 asciiValue = 16rFF]) ifTrue: 
		[self useLittleEndian: false.
		readBOM := true].

	readBOM ifTrue: 
		[self useByteOrderMark: true.
		character1 := aStream basicNext ifNil: [^nil]..
		character2 := aStream basicNext ifNil: [^nil]].

	charValue := self useLittleEndian 
		ifTrue: [(character2 charCode bitShift: 8) + character1 charCode] 
		ifFalse: [(character1 charCode bitShift: 8) + character2 charCode].

	^ self charFromStream: aStream withFirst: charValue.
! !

!UTF16TextConverter methodsFor: 'conversion' stamp: 'HenrikSperreJohansen 8/9/2010 15:01'!
nextPut: aCharacter toStream: aStream

	| v low high |
	
	self checkBOMOn: aStream.

	v := aCharacter charCode.
	v > 16rFFFF ifFalse: [
		self next16BitValue: v toStream: aStream.
		^ self.
	] ifTrue: [
		v := v - 16r10000.
		low := (v \\ 16r400) + 16rDC00.
		high := (v // 16r400) + 16rD800.
		self next16BitValue: high toStream: aStream.
		self next16BitValue: low toStream: aStream.
	]! !

!UTF16TextConverter methodsFor: 'conversion' stamp: 'HenrikSperreJohansen 8/9/2010 15:04'!
nextPutAll: aString toStream: aStream
	
	self checkBOMOn: aStream.
	^super nextPutAll: aString toStream: aStream! !

!UTF16TextConverter methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/10/2010 16:21'!
nextPutByteString: aString toStream: aStream
	| utf16Bytes start |
	"No bytestring character will be a surrogate pair, so simply write the 2x size string with the correct endianness"
	utf16Bytes := String new: aString size* 2.
	start := useLittleEndian ifTrue: [1] ifFalse: [2].
	start to: utf16Bytes size by: 2 do: [:ix | utf16Bytes at: ix put: (aString at: ix +1 // 2 )].
	aStream basicNextPutAll: utf16Bytes.
	! !

!UTF16TextConverter methodsFor: 'private' stamp: 'HenrikSperreJohansen 6/7/2010 14:28'!
swapLatin1EncodingByteOrder
	latin1Encodings := latin1Encodings collect: [:each | each reverse]! !

!UTF16TextConverter methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/9/2010 15:02'!
useByteOrderMark

	^ useByteOrderMark
! !

!UTF16TextConverter methodsFor: 'accessing' stamp: 'yo 1/12/2004 13:54'!
useByteOrderMark: aBoolean

	useByteOrderMark := aBoolean.
! !

!UTF16TextConverter methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/9/2010 15:02'!
useLittleEndian

	^ useLittleEndian
! !

!UTF16TextConverter methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 6/7/2010 14:16'!
useLittleEndian: aBoolean
	aBoolean = self useLittleEndian ifFalse: [self swapLatin1EncodingByteOrder].
	useLittleEndian := aBoolean.
! !

TextConverter subclass: #UTF8TextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!UTF8TextConverter commentStamp: '<historical>' prior: 0!
Text converter for UTF-8.  Since the BOM is used to distinguish the MacRoman code and UTF-8 code, BOM is written for UTF-8 by #writeBOMOn: which is called by client.!

UTF8TextConverter subclass: #UTF8DecomposedTextConverter
	instanceVariableNames: 'combinedChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-TextConversion'!

!UTF8DecomposedTextConverter commentStamp: 'michael.rueger 3/2/2009 13:51' prior: 0!
An UTF8DecomposedTextConverter converts from decomposed UTF8 using the UnicodeCompositionStream.

Instance Variables
!

!UTF8DecomposedTextConverter class methodsFor: 'utilities' stamp: 'michael.rueger 3/31/2009 16:38'!
encodingNames

	^Array new
! !

!UTF8DecomposedTextConverter methodsFor: 'conversion' stamp: 'michael.rueger 3/2/2009 14:12'!
nextFromStream: aStream

	| char resultChar |
	char := super nextFromStream: aStream.
	(CombinedChar isCompositionCharacter: char charCode)
		ifFalse: [^char].
	combinedChar
		ifNil: [
			combinedChar := CombinedChar new.
			combinedChar simpleAdd: char]
		ifNotNil: [
			[combinedChar simpleAdd: char]
				whileFalse: [char := super nextFromStream: aStream].
			resultChar := combinedChar combined.
			combinedChar := CombinedChar new.
			combinedChar simpleAdd: char.
			^resultChar]
! !

!UTF8TextConverter class methodsFor: 'utilities' stamp: 'yo 12/19/2003 22:01'!
encodingNames

	^ #('utf-8' 'utf8') copy.
! !

!UTF8TextConverter class methodsFor: 'accessing' stamp: 'tak 1/12/2005 13:22'!
writeBOMOn: aStream
	"Write Byte Order Mark"
	aStream nextPut: 16rEF.
	aStream nextPut: 16rBB.
	aStream nextPut: 16rBF.
! !

!UTF8TextConverter methodsFor: 'conversion' stamp: 'ar 4/6/2006 10:15'!
errorMalformedInput
	^self error: 'Invalid utf8 input detected'! !

!UTF8TextConverter methodsFor: 'friend' stamp: 'yo 11/8/2002 16:17'!
leadingChar

	^ self shouldNotImplement
! !

!UTF8TextConverter methodsFor: 'conversion' stamp: 'GabrielOmarCotelli 5/25/2010 21:22'!
nextFromStream: aStream
	| character1 value1 character2 value2 unicode character3 value3 character4 value4 |
	aStream isBinary
		ifTrue: [ ^ aStream basicNext ].
	character1 := aStream basicNext.
	character1 isNil
		ifTrue: [ ^ nil ].
	value1 := character1 asciiValue.
	value1 <= 127
		ifTrue: [ 
			"1-byte character"
			^ character1 ].	"at least 2-byte character"
	character2 := aStream basicNext.
	character2 isNil
		ifTrue: [ ^ self errorMalformedInput ].
	value2 := character2 asciiValue.
	(value1 bitAnd: 16rE0) = 192
		ifTrue: [ ^ Unicode value: ((value1 bitAnd: 31) bitShift: 6) + (value2 bitAnd: 63) ].	"at least 3-byte character"
	character3 := aStream basicNext.
	character3 isNil
		ifTrue: [ ^ self errorMalformedInput ].
	value3 := character3 asciiValue.
	(value1 bitAnd: 16rF0) = 224
		ifTrue: [ unicode := ((value1 bitAnd: 15) bitShift: 12) + ((value2 bitAnd: 63) bitShift: 6) + (value3 bitAnd: 63) ].
	(value1 bitAnd: 16rF8) = 240
		ifTrue: [ 
			"4-byte character"
			character4 := aStream basicNext.
			character4 isNil
				ifTrue: [ ^ self errorMalformedInput ].
			value4 := character4 asciiValue.
			unicode := ((value1 bitAnd: 16r7) bitShift: 18) + ((value2 bitAnd: 63) bitShift: 12)
				+ ((value3 bitAnd: 63) bitShift: 6) + (value4 bitAnd: 63) ].
	unicode isNil
		ifTrue: [ ^ self errorMalformedInput ].
	unicode > 16r10FFFD
		ifTrue: [ ^ self errorMalformedInput ].
	unicode = 16rFEFF
		ifTrue: [ ^ self nextFromStream: aStream ].
	^ Unicode value: unicode! !

!UTF8TextConverter methodsFor: 'conversion' stamp: 'ar 4/9/2005 22:29'!
nextPut: aCharacter toStream: aStream 
	| leadingChar nBytes mask shift ucs2code |
	aStream isBinary ifTrue: [^aCharacter storeBinaryOn: aStream].
	leadingChar := aCharacter leadingChar.
	(leadingChar = 0 and: [aCharacter asciiValue < 128]) ifTrue: [
		aStream basicNextPut: aCharacter.
		^ aStream.
	].

	"leadingChar > 3 ifTrue: [^ aStream]."

	ucs2code := aCharacter asUnicode.
	ucs2code ifNil: [^ aStream].

	nBytes := ucs2code highBit + 3 // 5.
	mask := #(128 192 224 240 248 252 254 255) at: nBytes.
	shift := nBytes - 1 * -6.
	aStream basicNextPut: (Character value: (ucs2code bitShift: shift) + mask).
	2 to: nBytes do: [:i | 
		shift := shift + 6.
		aStream basicNextPut: (Character value: ((ucs2code bitShift: shift) bitAnd: 63) + 128).
	].

	^ aStream.
! !
ByteTextConverter initialize!
ISO88597TextConverter initialize!
ISO88592TextConverter initialize!
KOI8RTextConverter initialize!
ISO885915TextConverter initialize!
MacRomanTextConverter initialize!
CP1253TextConverter initialize!
CP1250TextConverter initialize!
MultiByteFileStream initialize!
CP1252TextConverter initialize!
